import scipy.stats as st


def sterling2(n: int,k: int) -> int:
  if k==1: return 1
  if n==k: return 1
  else: return(k*sterling2(n-1,k)+sterling2(n-1,k-1))

def P(n:int,k:int)-> int:#integer k-partition
  if n<1 or k<1: return 0
  if k==1: return 1
  if n==k: return 1
  else: 
    return(P(n-1,k-1)+P(n-k,k))

def factorial(n:int)->int:
  if n==0: return 1
  c=1
  for i in range(1,n+1): c=c*i
  return c

def sterling1(n:int,k:int)->int:
  if k==1: return factorial(n-1)
  if n==k: return 1
  else: return((n-1)*sterling1(n-1,k)+sterling1(n-1,k-1))

def C(n:int,k:int) -> int: #n choose k
  return(factorial(n)/factorial(k)/factorial(n-k))

def mulC(n:int,*a:int) -> int: #divide n to subsets of a1, a2, ...
  if sum(a) != n:
    return()
  c=factorial(n)
  for i in a:
    c=c//factorial(i)
  return(c)
  
def partition (n: int, k: int, subset_dif: bool, element_dif: bool) -> int:
  if subset_dif == 1:
    if element_dif == 1:
      return(sterling2(n,k)*factorial(k))
    if element_dif == 0:
      return(sterling2(n,k))
  if subset_dif == 0:
    if element_dif == 1:
      return(C(n-1,n-k))
    if element_dif == 0:
      return(P(n,k))
    
def partition_object(n: int, k: int, obj: chr):
  if obj == "s": return(sterling2(n,k))
  if obj == "l": return(C(n-1,k-1)*factorial(n)/factorial(k))
  if obj == "c": return(sterling1(n,k))
    
def derangement(n: int):
  if n == 0: return 1
  if n%2 == 0:
    return(derangement(n-1)*n +1)
  else: 
    return(derangement(n-1)*n -1)

#probability
def mean_estimator(a: list) -> int:
  return(sum(a)/len(a))

def SD_estimator(a: list) -> int:
  return((sum([(x-mean_estimator(a))**2 for x in a])/(len(a)-1)) ** (1/2))
# a=[85.6,95.4,91.3,95.2,84.5,94.5,85.5,89.5,95.5,76.5,98.7,84.2,95.4,83.2,97.5]

#input data with n obs, with unknown distribution family + mean + SD, 
#output confidence interval of mean and std
#case: n<30 (unreliable)
#case n>=30 (more reliable - CLT theorem)
def CI(a: list, alpha: int) -> list [tuple, tuple]:
  n=len(a)
  mu = mean_estimator(a)
  SD = SD_estimator(a)
  if n < 30:
    lower_mu = float(mu - st.t.ppf(1 - alpha/2,df=4-1)*SD/(n**(1/2)))
    upper_mu = float(mu + st.t.ppf(1 - alpha/2,df=4-1)*SD/(n**(1/2)))
    lower_sd = float((n*SD*SD/st.chi2.ppf(1 - alpha/2,df=n-1))**(1/2))
    upper_sd = float((n*SD*SD/st.chi2.ppf(alpha/2,df=n-1))**(1/2))
    return([(lower_mu,upper_mu),(lower_sd,upper_sd)])
  if n >= 30:
    u=st.norm.ppf(1 - alpha/2)
    lower_mu = float(mu - u*SD/(n**(1/2)))
    upper_mu = float(mu + u*SD/(n**(1/2)))
    lower_sd = float(SD/(1+u/((2*n)**(1/2))))
    upper_sd = float(SD/(1-u/((2*n)**(1/2))))
    return([(lower_mu,upper_mu),(lower_sd,upper_sd)])

# We designed a game with EV = 0.97 and STD = 13. The game
# costs 1 coin to play. What is the probability that after 10,000
# games the company will be losing money?
def losing_prob(ev:float, std:float, n:int, cost_per_play = float) -> float:
  m = n*cost_per_play
  return (1- st.norm.cdf( (m-n*ev)/(std*(n**0.5)) ))
# print(losing_prob(ev=0.97, std=13, n = 10000, cost_per_play=1))

