
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as stats

def vis_SND(muy, sigma, n_sigma=5):
    x = np.linspace(muy - n_sigma*sigma, muy + n_sigma*sigma, 1000) 
    y = stats.norm.pdf(x, muy, sigma)

    plt.figure(figsize=(10, 6))
    plt.plot(x, y, color='royalblue', linewidth=3, label="Normal Distribution")

    for i, color in zip([1, 2, 3], ["#ff9999", "#99ccff", "#99ff99"]):
        plt.fill_between(x, y, where=(x >= muy - i*sigma) & (x <= muy - (i-1)*sigma), color=color, alpha=0.3, label=f'±{i}sigma')
        plt.fill_between(x, y, where=(x >= muy + (i-1)*sigma) & (x <= muy + i*sigma), color=color, alpha=0.3)

    plt.title("Normal Distribution (Gaussian Bell Curve)", fontsize=16)
    plt.xlabel("X", fontsize=12)
    plt.ylabel("Probability density", fontsize=12)
    plt.axvline(muy, color='black', linestyle='dashed', linewidth=1)
    plt.legend()
    plt.grid(alpha=0.5)
    plt.show()


def vis_ND():
    _, axes = plt.subplots(2, 1, figsize=(16,8))

    n_sigma = 5
    sigma_base = 1
    sigma_step = 0.01
    # muy_values = np.arange(-2, 2.5, 0.5)
    muy_values = [0]
    sigma_values = np.arange(0.2, sigma_base+sigma_step, sigma_step)

    for i, muy in enumerate(muy_values):
        for sigma in sigma_values:
            x = np.linspace(muy - n_sigma*sigma, muy + n_sigma*sigma, 1000) 
            y_pdf = stats.norm.pdf(x, muy, sigma)
            y_cdf = stats.norm.cdf(x, muy, sigma)
            axes[0].plot(x, y_pdf, linewidth=1, alpha=0.7)
            axes[1].plot(x, y_cdf, linewidth=1, alpha=0.7)

    plt.grid(alpha=0.5)
    plt.show()


def vis_BD():
    fig, axes=plt.subplots(2, 1, figsize=(16,6))
    n_values = np.arange(0, 1001, 500)
    # n_values = [500, 1000]
    p_values = np.arange(0, 1, 0.01)
    x = np.arange(0, 1100, 1)
    
    for n in n_values:
        for p in p_values:
            y_pdf = stats.binom.pmf(x, n, p)
            y_cdf = stats.binom.cdf(x, n, p)
            axes[0].plot(x, y_pdf, linewidth=1, alpha=0.7)
            axes[1].plot(x, y_cdf, linewidth=1, alpha=0.7)

    plt.grid(alpha=0.5)
    plt.show()


def vis_GD():
    fig, axes=plt.subplots(2, 1, figsize=(16,6))
    p_values = np.arange(0, 1, 0.1)
    x = np.arange(0, 400, 1)
    
    for p in p_values:
        y_pdf = stats.geom.pmf(x, p)
        y_cdf = stats.geom.cdf(x, p)
        axes[0].plot(x, y_pdf, linewidth=1, alpha=0.7)
        axes[1].plot(x, y_cdf, linewidth=1, alpha=0.7)

    plt.grid(alpha=0.5)
    plt.show()


def vis_PD():
    fig, axes=plt.subplots(2, 1, figsize=(16,6))
    lam_values = np.arange(1, 400, 10)
    x = np.arange(0, 500, 1)
    
    for lam in lam_values:
        y_pdf = stats.poisson.pmf(x, lam)
        y_cdf = stats.poisson.cdf(x, lam)
        axes[0].plot(x, y_pdf, linewidth=1, alpha=0.7)
        axes[1].plot(x, y_cdf, linewidth=1, alpha=0.7)

    plt.grid(alpha=0.5)
    plt.show()
    

def vis_ED():
    fig, axes=plt.subplots(2, 1, figsize=(16,6))
    lam_values = np.arange(0, 1.2, 0.1)
    x = np.arange(0, 500, 1)
    
    for lam in lam_values:
        y_pdf = stats.expon.pdf(x, 0, 1/lam)
        y_cdf = stats.expon.cdf(x, 0, 1/lam)
        axes[0].plot(x, y_pdf, linewidth=1, alpha=0.7)
        axes[1].plot(x, y_cdf, linewidth=1, alpha=0.7)

    plt.grid(alpha=0.5)
    plt.show()
    

def vis_ParetoD():
    fig, axes=plt.subplots(2, 1, figsize=(16,6))
    alpha_values = np.arange(0, 5, 0.1)
    x = np.arange(0, 100, 1)
    
    for alpha in alpha_values:
        y_pdf = stats.pareto.pdf(x, alpha)
        y_cdf = stats.pareto.cdf(x, alpha)
        axes[0].plot(x, y_pdf, linewidth=1, alpha=0.7)
        axes[1].plot(x, y_cdf, linewidth=1, alpha=0.7)

    plt.grid(alpha=0.5)
    plt.show()
